@@ -76,7 +76,8 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
     # Version 1.33: Add retired and retired_reason fields
     # Version 1.34: Add lessee field
     # Version 1.35: Add network_data field
-    VERSION = '1.35'
+    # Version 1.36: Add boot_mode and secure_boot fields
+    VERSION = '1.36'
 
     dbapi = db_api.get_instance()
 
@@ -166,6 +167,8 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         'retired': objects.fields.BooleanField(nullable=True),
         'retired_reason': object_fields.StringField(nullable=True),
         'network_data': object_fields.FlexibleDictField(nullable=True),
+        'boot_mode': object_fields.StringField(nullable=True),
+        'secure_boot': object_fields.BooleanField(nullable=True),
     }
 
     def as_dict(self, secure=False, mask_configdrive=True):
@@ -174,11 +177,12 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
             d['driver_info'] = strutils.mask_dict_password(
                 d.get('driver_info', {}), "******")
             iinfo = d.pop('instance_info', {})
-            if not mask_configdrive:
-                configdrive = iinfo.pop('configdrive', None)
+            configdrive = iinfo.pop('configdrive', None)
             d['instance_info'] = strutils.mask_dict_password(iinfo, "******")
-            if not mask_configdrive and configdrive:
-                d['instance_info']['configdrive'] = configdrive
+            if configdrive is not None:
+                d['instance_info']['configdrive'] = (
+                    "******" if mask_configdrive else configdrive
+                )
             d['driver_internal_info'] = strutils.mask_dict_password(
                 d.get('driver_internal_info', {}), "******")
         return d
@@ -212,12 +216,14 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
             raise exception.InvalidParameterValue(msg)
 
     def _set_from_db_object(self, context, db_object, fields=None):
-        fields = set(fields or self.fields) - {'traits'}
-        super(Node, self)._set_from_db_object(context, db_object, fields)
-        self.traits = object_base.obj_make_list(
-            context, objects.TraitList(context),
-            objects.Trait, db_object['traits'])
-        self.traits.obj_reset_changes()
+        use_fields = set(fields or self.fields) - {'traits'}
+        super(Node, self)._set_from_db_object(context, db_object, use_fields)
+        if not fields or 'traits' in fields:
+            self.traits = object_base.obj_make_list(
+                context, objects.TraitList(context),
+                objects.Trait, db_object['traits'],
+                fields=['trait', 'version'])
+            self.traits.obj_reset_changes()
 
     # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
     # methods can be used in the future to replace current explicit RPC calls.
@@ -312,7 +318,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
     # @object_base.remotable_classmethod
     @classmethod
     def list(cls, context, limit=None, marker=None, sort_key=None,
-             sort_dir=None, filters=None):
+             sort_dir=None, filters=None, fields=None):
         """Return a list of Node objects.
 
         :param cls: the :class:`Node`
@@ -322,13 +328,29 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         :param sort_key: column to sort results by.
         :param sort_dir: direction to sort. "asc" or "desc".
         :param filters: Filters to apply.
+        :param fields: Requested fields to be returned. Please note, some
+                       fields are mandatory for the data model and are
+                       automatically included. These are: id, version,
+                       updated_at, created_at, owner, and lessee.
         :returns: a list of :class:`Node` object.
-
         """
+        if fields:
+            # All requests must include version, updated_at, created_at
+            # owner, and lessee to support access controls and database
+            # version model updates. Driver and conductor_group are required
+            # for conductor mapping.
+            target_fields = ['id'] + fields[:] + ['version', 'updated_at',
+                                                  'created_at', 'owner',
+                                                  'lessee', 'driver',
+                                                  'conductor_group']
+        else:
+            target_fields = None
+
         db_nodes = cls.dbapi.get_node_list(filters=filters, limit=limit,
                                            marker=marker, sort_key=sort_key,
-                                           sort_dir=sort_dir)
-        return cls._from_db_object_list(context, db_nodes)
+                                           sort_dir=sort_dir,
+                                           fields=target_fields)
+        return cls._from_db_object_list(context, db_nodes, target_fields)
 
     # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
     # methods can be used in the future to replace current explicit RPC calls.
@@ -625,6 +647,8 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
             be set to None or removed.
         Version 1.35: network_data was added. For versions prior to this, it
             should be set to empty dict (or removed).
+        Version 1.36: boot_mode, secure_boot were was added. Defaults are None.
+            For versions prior to this, it should be set to None or removed.
 
         :param target_version: the desired version of the object
         :param remove_unavailable_fields: True to remove fields that are
@@ -639,7 +663,8 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
                   ('bios_interface', 24), ('fault', 25),
                   ('automated_clean', 28), ('protected_reason', 29),
                   ('owner', 30), ('allocation_id', 31), ('description', 32),
-                  ('retired_reason', 33), ('lessee', 34)]
+                  ('retired_reason', 33), ('lessee', 34), ('boot_mode', 36),
+                  ('secure_boot', 36)]
 
         for name, minor in fields:
             self._adjust_field_to_version(name, None, target_version,
@@ -681,6 +706,8 @@ class NodePayload(notification.NotificationPayloadBase):
         'description': ('node', 'description'),
         'driver': ('node', 'driver'),
         'extra': ('node', 'extra'),
+        'boot_mode': ('node', 'boot_mode'),
+        'secure_boot': ('node', 'secure_boot'),
         'inspection_finished_at': ('node', 'inspection_finished_at'),
         'inspection_started_at': ('node', 'inspection_started_at'),
         'instance_uuid': ('node', 'instance_uuid'),
@@ -735,7 +762,8 @@ class NodePayload(notification.NotificationPayloadBase):
     # Version 1.13: Add description field.
     # Version 1.14: Add retired and retired_reason fields exposed via API.
     # Version 1.15: Add node lessee field.
-    VERSION = '1.15'
+    # Version 1.16: Add boot_mode and secure_boot fields.
+    VERSION = '1.16'
     fields = {
         'clean_step': object_fields.FlexibleDictField(nullable=True),
         'conductor_group': object_fields.StringField(nullable=True),
@@ -745,6 +773,8 @@ class NodePayload(notification.NotificationPayloadBase):
         'description': object_fields.StringField(nullable=True),
         'driver': object_fields.StringField(nullable=True),
         'extra': object_fields.FlexibleDictField(nullable=True),
+        'boot_mode': object_fields.StringField(nullable=True),
+        'secure_boot': object_fields.BooleanField(nullable=True),
         'inspection_finished_at': object_fields.DateTimeField(nullable=True),
         'inspection_started_at': object_fields.DateTimeField(nullable=True),
         'instance_uuid': object_fields.UUIDField(nullable=True),
@@ -824,7 +854,8 @@ class NodeSetPowerStatePayload(NodePayload):
     # Version 1.13: Parent NodePayload version 1.13
     # Version 1.14: Parent NodePayload version 1.14
     # Version 1.15: Parent NodePayload version 1.15
-    VERSION = '1.15'
+    # Version 1.16: Parent NodePayload version 1.16
+    VERSION = '1.16'
 
     fields = {
         # "to_power" indicates the future target_power_state of the node. A
@@ -880,7 +911,8 @@ class NodeCorrectedPowerStatePayload(NodePayload):
     # Version 1.13: Parent NodePayload version 1.13
     # Version 1.14: Parent NodePayload version 1.14
     # Version 1.15: Parent NodePayload version 1.15
-    VERSION = '1.15'
+    # Version 1.16: Parent NodePayload version 1.16
+    VERSION = '1.16'
 
     fields = {
         'from_power': object_fields.StringField(nullable=True)
@@ -921,7 +953,8 @@ class NodeSetProvisionStatePayload(NodePayload):
     # Version 1.14: Parent NodePayload version 1.14
     # Version 1.15: Parent NodePayload version 1.15
     # Version 1.16: add driver_internal_info
-    VERSION = '1.16'
+    # Version 1.17: Parent NodePayload version 1.16
+    VERSION = '1.17'
 
     SCHEMA = dict(NodePayload.SCHEMA,
                   **{'instance_info': ('node', 'instance_info'),
@@ -970,7 +1003,8 @@ class NodeCRUDPayload(NodePayload):
     # Version 1.11: Parent NodePayload version 1.13
     # Version 1.12: Parent NodePayload version 1.14
     # Version 1.13: Parent NodePayload version 1.15
-    VERSION = '1.13'
+    # Version 1.14: Parent NodePayload version 1.16
+    VERSION = '1.14'
 
     SCHEMA = dict(NodePayload.SCHEMA,
                   **{'instance_info': ('node', 'instance_info'),
