@@ -13,6 +13,7 @@
 from unittest import mock
 
 import eventlet
+from keystoneauth1 import exceptions as ks_exception
 import openstack
 
 from ironic.common import context
@@ -61,6 +62,17 @@ class GetClientTestCase(db_base.DbTestCase):
         self.assertEqual(1, mock_auth.call_count)
         self.assertEqual(1, mock_session.call_count)
 
+    def test__get_client_connection_problem(
+            self, mock_conn, mock_session, mock_auth):
+        mock_conn.side_effect = ks_exception.DiscoveryFailure("")
+        self.assertRaises(exception.ConfigInvalid,
+                          inspector._get_client, self.context)
+        mock_conn.assert_called_once_with(
+            session=mock.sentinel.session,
+            oslo_conf=mock.ANY)
+        self.assertEqual(1, mock_auth.call_count)
+        self.assertEqual(1, mock_session.call_count)
+
 
 class BaseTestCase(db_base.DbTestCase):
     def setUp(self):
@@ -123,11 +135,6 @@ class InspectHardwareTestCase(BaseTestCase):
     def test_validate_ok(self, mock_client):
         self.iface.validate(self.task)
 
-    def test_validate_invalid_kernel_params(self, mock_client):
-        CONF.set_override('extra_kernel_params', 'abcdef', group='inspector')
-        self.assertRaises(exception.InvalidParameterValue,
-                          self.iface.validate, self.task)
-
     @mock.patch.object(redfish_utils, 'get_system', autospec=True)
     @mock.patch.object(inspect_utils, 'create_ports_if_not_exist',
                        autospec=True)
@@ -231,7 +238,7 @@ class InspectHardwareTestCase(BaseTestCase):
                                    mock_client):
         CONF.set_override('extra_kernel_params',
                           'ipa-inspection-collectors=default,logs '
-                          'ipa-collect-dhcp=1',
+                          'ipa-collect-dhcp=1 something',
                           group='inspector')
         endpoint = 'http://192.169.0.42:5050/v1'
         mock_client.return_value.get_endpoint.return_value = endpoint
@@ -246,6 +253,7 @@ class InspectHardwareTestCase(BaseTestCase):
                 'ipa-inspection-callback-url': endpoint + '/continue',
                 'ipa-inspection-collectors': 'default,logs',
                 'ipa-collect-dhcp': '1',
+                'something': None,
             })
         self.driver.network.add_inspection_network.assert_called_once_with(
             self.task)
